/*
 * blueMail offline mail reader
 * bmail class

 Copyright (c) 1996 Toth Istvan <stoty@vma.bme.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>,
                    Robert Vukovic <vrobert@uns.ns.ac.yu>
 Copyright (c) 1998 Ingo Brueckl <ib@wupperonline.de>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#include "bmail.h"
#include "compress.h"

const char *net_address::text()
{
        static char netText[25];

        if (point)
                sprintf(netText, "%u:%u/%u.%u", zone, net, node, point);
        else
                sprintf(netText, "%u:%u/%u", zone, net, node);

        return netText;
}

void net_address::init(const char *source)
{
        if (sscanf(source, "%u:%u/%u.%u", &zone, &net, &node, &point) == 3)
                point = 0;
}

bmail::bmail()
{
        resourceObject = new resource();
};

bmail::~bmail()
{
        delete resourceObject;
};

void bmail::Delete()
{
        delete areaList;
        delete driverList;
        delete workList;
}

int bmail::selectPacket(const char *packetName)
{
        char fname[255];

        resourceObject->set(PacketName, packetName);

        // Uncompression is done here
        sprintf(fname, "%s/%s", resourceObject->get(PacketDir),
                packetName);

        if (!resourceObject->get(oldPacketName) ||
            strcmp(packetName, resourceObject->get(oldPacketName))) {
                resourceObject->set(oldPacketName, packetName);
                if (uncompressFile(resourceObject, fname,
                    resourceObject->get(PacketWorkDir)))
                        return PKT_UNCOMP_FAILED;
        }

        workList = new file_list(resourceObject->get(PacketWorkDir));
        driverList = new driver_list(this);

        if (!driverList->getNoOfDrivers()) {
                delete driverList;
                delete workList;
                return PKT_TYPE_UNKNOWN;
        }

        areaList = new area_list(this);
        return 0;
};

void bmail::saveRead()
{
        driverList->getReadObject(driverList->getDriver(REPLY_AREA + 1))->
                saveXTI();
};
