/*
 * blueMail offline mail reader
 * main_read_class, reply_read_class

 Copyright (c) 1996 Toth Istvan <stoty@vma.bme.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>
 Copyright (c) 1998 Ingo Brueckl <ib@wupperonline.de>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#include "bmail.h"
#include "compress.h"

/* read_class -- virtual */

read_class::read_class()
{
}

read_class::~read_class()
{
}

/* main_read_class -- for regular areas */

main_read_class::main_read_class (bmail *bm, specific_driver *driver)
{
  FILE *readFile;
  char *readFileName;

  ro = bm->resourceObject;

  noOfAreas = driver->getNoOfAreas();
  noOfLetters = new int[noOfAreas];
  xti = new XTI_REC *[noOfAreas];

  readFileName = readFilePath(ro->get(PacketName));
  readFile = bm->workList->ftryopen(readFileName, "rb");

  for (int a = 0; a < noOfAreas; a++)
  {
    driver->selectArea(a);
    noOfLetters[a] = driver->getNoOfLetters();
    xti[a] = new XTI_REC[noOfLetters[a]];
    if (readFile)
      for (int l = 0; l < noOfLetters[a]; l++)
        fscanf(readFile, "%c%c", &xti[a][l].flags, &xti[a][l].marks);
  }

  if (readFile) fclose(readFile);
  else driver->initXTI(xti);
}

main_read_class::~main_read_class ()
{
  while(noOfAreas) delete[] xti[--noOfAreas];
  delete[] xti;
  delete[] noOfLetters;
}

void main_read_class::setRead (int area, int letter, int value)
{
  if (value) xti[area][letter].flags |= XTI_HAS_READ;
  else xti[area][letter].flags &= ~XTI_HAS_READ;
}

int main_read_class::getRead (int area, int letter)
{
  return xti[area][letter].flags & XTI_HAS_READ;
}

void main_read_class::setStatus (int area, int letter, tBYTE flags, tBYTE marks)
{
  xti[area][letter].flags = flags;
  xti[area][letter].marks = marks;
}

const XTI_REC *main_read_class::getStatus (int area, int letter)
{
  return &xti[area][letter];
}

int main_read_class::getNoOfUnread (int area)
{
  int tmp = 0;

  for (int l = 0; l < noOfLetters[area]; l++)
    if ((xti[area][l].flags & XTI_HAS_READ) == 0) tmp++;

  return tmp;
}

void main_read_class::saveXTI ()
{
  FILE *readFile;
  const char *readFileName;

  readFileName = readFilePath(ro->get(PacketName));
  readFile = fopen(readFileName, "wb");

  if (readFile)
  {
    for (int a = 0; a < noOfAreas; a++)
      for (int l = 0; l < noOfLetters[a]; l++)
        fprintf(readFile, "%c%c", xti[a][l].flags, xti[a][l].marks);

    fclose(readFile);

    /* the .xti file to the packet */
    compressAddFile(ro, ro->get(PacketDir), ro->get(PacketName), readFileName);
  }
}

char *main_read_class::readFilePath (const char *FileName)
{
  static char tmp[13];

  if(mychdir(ro->get(PacketWorkDir)))
    fatalError("Could not change to packet work dir.");

  sprintf(tmp, "%.8s.xti", findBaseName(FileName));
  return tmp;
}

/* reply_read_class -- for reply areas */

/* (Formerly known as dummy_read_class, because it does almost nothing.
What it does is mainly to suppress compiler warnings.) */

reply_read_class::reply_read_class (bmail *bmA, specific_driver *driverA)
{
  bmA = bmA;
  driverA = driverA;
}

reply_read_class::~reply_read_class ()
{
}

void reply_read_class::setRead (int area, int letter, int value)
{
  area = area;
  letter = letter;
  value = value;
}

int reply_read_class::getRead (int area, int letter)
{
  area = area;
  letter = letter;
  return XTI_HAS_READ;
}

void reply_read_class::setStatus (int area, int letter, tBYTE flags, tBYTE marks)
{
  area = area;
  letter = letter;
  flags = flags;
  marks = marks;
}

const XTI_REC *reply_read_class::getStatus (int area, int letter)
{
  static XTI_REC dummy = {XTI_HAS_READ, 0};

  area = area;
  letter = letter;
  return &dummy;
}

int reply_read_class::getNoOfUnread (int area)
{
  area = area;
  return 0;
}

void reply_read_class::saveXTI ()
{
}
