/*
 * blueMail offline mail reader
 * resource class

 Copyright (c) 1996 Toth Istvan <stoty@vma.bme.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>
 Copyright (c) 1998 Ingo Brueckl <ib@wupperonline.de>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#ifndef RESOURCE_H
#define RESOURCE_H

#include <stdio.h>

#ifdef __MSDOS__
  #define DEFEDIT "edit"
  #define DEFARJ "arj a -e"
  #define DEFUNARJ "arj e"
  #define DEFZIP "pkzip"
  #define DEFUNZIP "pkunzip"
  #define DEFLHA "lha a"
  #define DEFUNLHA "lha e"
#else
  #ifdef __EMX__
    #define DEFEDIT "tedit"
  #else
    #define DEFEDIT "vi"
  #endif
  #define DEFUNARJ "unarj e"
  #define DEFZIP "zip -j"
  #define DEFUNZIP "unzip -j -o -L"
  #define DEFLHA "lha a"
  #define DEFUNLHA "lha efi"
#endif

#if defined(__MSDOS__) || defined(__EMX__)
  #define RCNAME "bmail.rc"
  #define ADDRBOOK "address.bk"
#else
  #define RCNAME ".bmailrc"
  #define ADDRBOOK "addressbook"
#endif

enum OptionIDs
{
  dirFIRST,
    homeDir = dirFIRST,
    bmailDir,
    PacketDir,
    ReplyDir,
    SaveDir,
    WorkDir,
  dirLAST = WorkDir,
  stringFIRST,
    UserName = stringFIRST,
  stringLAST = UserName,
  internalFIRST,                                 /* internally used options */
    PacketWorkDir = internalFIRST,
    ReplyWorkDir,
    PacketName,
    oldPacketName,
    BBSName,
    SysOpName,
    LoginName,
    AliasName,
  internalLAST = AliasName,
  cmdFIRST,
    arjUncompressCommand = cmdFIRST,
    zipUncompressCommand,
    lhaUncompressCommand,
    unknownUncompressCommand,
    arjCompressCommand,
    zipCompressCommand,
    lhaCompressCommand,
    unknownCompressCommand,
    editor,
  cmdLAST = editor,
  fileFIRST,
    AddressFile = fileFIRST,
    TaglineFile,
    sigFile,
  fileLAST = sigFile,
  miscFIRST,
    SupprAreaLInfo = miscFIRST,
    SupprLineCounter,
    LSAreaList,
    LSLetterList,
    FullLetterList,
    SkipLetterList,
    SaveLastreadPointers,
  miscLAST = SaveLastreadPointers,
  colorFIRST,        /* colors must be in same order as default color array */
    colorMainBack = colorFIRST,
    colorMainBorder,
    colorMainBottSep,
    colorHelpDescr,
    colorHelpKeys,
    colorHelpBorder,
    colorHelpText,
    colorWelcBorder,
    colorWelcHeader,
    colorWelcText,
    colorAddrList,
    colorAddrBorder,
    colorAddrTText,
    colorAddrHeader,
    colorAddrKeys,
    colorAddrDescr,
    colorAddrInp,
    colorWarnText,
    colorWarnKeys,
    colorLetterListUnread,
    colorLetterListRead,
    colorLetterLBorder,
    colorLetterLTText,
    colorLetterLArea,
    colorLetterLHeader,
    colorLetterLFromUser,
    colorLetterLToUser,
    colorSaveBoxBorder,
    colorSaveBoxHeader,
    colorSaveBoxInp,
    colorLetterText,
    colorLetterQText,
    colorLetterTagline,
    colorLetterTearline,
    colorLetterOrigin,
    colorLetterBottLine,
    colorLetterHText,
    colorLetterHMsgNum,
    colorLetterHFrom,
    colorLetterHTo,
    colorLetterHSubject,
    colorLetterHDate,
    colorLetterHBorder,
    colorLetterHFlags,
    colorLetterHFlagsHi,
    colorPacketList,
    colorPacketLBorder,
    colorPacketLHeader,
    colorLiAreaList,
    colorLiAreaBorder,
    colorLiAreaHeader,
    colorAreaListUnread,
    colorAreaListRead,
    colorAreaLBorder,
    colorAreaLTText,
    colorAreaLHeader,
    colorAreaLReply,
    colorAreaLInfoDescr,
    colorAreaLInfoText,
    colorRepBoxBorder,
    colorRepBoxDescr,
    colorRepBoxText,
    colorRepBoxInp,
    colorTagBoxList,
    colorTagBoxBorder,
    colorTagBoxTText,
    colorTagBoxKeys,
    colorTagBoxDescr,
    colorTagBoxInp,
    colorShadow,
  colorLAST = colorShadow,
  maxOptions = colorLAST + 1     /* No. of last ID + 1 */
};

const char *const OptionNames[maxOptions] =
{
  "home",
  "bmail",
  "packet",
  "reply",
  "save",
  "work",
  "UserName",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "arjUncompress",
  "zipUncompress",
  "lhaUncompress",
  "unknownUncompress",
  "arjCompress",
  "zipCompress",
  "lhaCompress",
  "unknownCompress",
  "editor",
  "addressbook",
  "taglines",
  "signature",
  "SuppressAreaListInfo",
  "SuppressLineCounter",
  "LongAreaList",
  "LongLetterList",
  "FullsizeLetterList",
  "SkipLetterList",
  "SaveLastreadPointers",
  "MainBackground",
  "MainBorder",
  "MainBottomSeparator",
  "HelpDescription",
  "HelpKeys",
  "HelpBorder",
  "HelpText",
  "WelcomeBorder",
  "WelcomeHeader",
  "WelcomeText",
  "AddressbookList",
  "AddressbookBorder",
  "AddressbookTopText",
  "AddressbookHeader",
  "AddressbookKeys",
  "AddressbookDescription",
  "AddressbookInput",
  "WarningText",
  "WarningKeys",
  "LetterListUnread",
  "LetterListRead",
  "LetterListBorder",
  "LetterListTopText",
  "LetterListArea",
  "LetterListHeader",
  "LetterListFromUser",
  "LetterListToUser",
  "SaveBoxBorder",
  "SaveBoxHeader",
  "SaveBoxInput",
  "LetterText",
  "LetterQuotedText",
  "LetterTagline",
  "LetterTearline",
  "LetterOrigin",
  "LetterBottomline",
  "LetterHeaderText",
  "LetterHeaderMsgnum",
  "LetterHeaderFrom",
  "LetterHeaderTo",
  "LetterHeaderSubject",
  "LetterHeaderDate",
  "LetterHeaderBorder",
  "LetterHeaderFlags",
  "LetterHeaderFlagsHigh",
  "PacketList",
  "PacketListBorder",
  "PacketListHeader",
  "LittleAreaList",
  "LittleAreaListBorder",
  "LittleAreaListHeader",
  "AreaListUnread",
  "AreaListRead",
  "AreaListBorder",
  "AreaListTopText",
  "AreaListHeader",
  "AreaListReply",
  "AreaListInfoDescription",
  "AreaListInfoText",
  "ReplyBoxBorder",
  "ReplyBoxDescription",
  "ReplyBoxText",
  "ReplyBoxInput",
  "TaglineBoxList",
  "TaglineBoxBorder",
  "TaglineBoxTopText",
  "TaglineBoxKeys",
  "TaglineBoxDescription",
  "TaglineBoxInput",
  "Shadow",
};

class resource
{
  struct resource_struct
  {
    char *value;
  } resourceData[maxOptions];

  const char *fixPath(const char *);
  bool checkPath(const char *, bool);
  bool taglineCheck();
  bool verifyPaths();
  void createConfigBlock(FILE *, const int, const int, const char *);
  bool createNewConfig(const char *);
  void parseConfig(FILE *);
  void bmEachInit(int, const char *);
  void subWorkPath(int, const char *);
  void initinit();
  void workinit();
  void bmHomeInit();

public:

  resource();
  ~resource();
  const char *get(int) const;
  void set(int, const char *);
  bool isYes(int);
};

#endif
