/*
 * blueMail offline mail reader
 * address book

 Copyright (c) 1996 Kolossvary Tamas <thomas@vma.bme.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>
 Copyright (c) 1998 Ingo Brueckl <ib@wupperonline.de>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#include "interfac.h"

Person::Person()
{
        next = NULL;
        netmail_addr.point = 0;
}

AddressBook::AddressBook ()
{
  NumOfPersons = 0;
  NoEnter = 1;
}

void AddressBook::MakeActive ()
{
  const int e = interface->endpos(6, 20) + 1;

  people = NULL;

  list_max_y = LINES - 12;
  list_max_x = COLS - 6;

  list = new InfoWin(LINES - 5, COLS - 4, 2, 2, C_ADDRBORDER, "Addressbook",
                     C_ADDRTOPTEXT, 7);

  list->attrib(C_ADDRBORDER);
  list->horizline(list_max_y + 2, list_max_x);
  list->attrib(C_ADDRHEADER);
  list->putstring(1, 2, "Name                            Netmail address");

  list->attrib(C_ADDRKEYS);
  list->putstring(list_max_y + 5, e, "Q");
  list->attrib(C_ADDRDESCR);
  list->putstring(list_max_y + 5, e + 1, ": Leave Addressbook");

  inletter = ((interface->prevactive() == letter) ||
              (interface->prevactive() == littlearealist)) &&
              !bm.areaList->isReplyArea();

  if (inletter)
  {
    list->attrib(C_ADDRKEYS);
    list->putstring(list_max_y + 3, 2, "P");
    list->attrib(C_ADDRDESCR);
    list->putstring(list_max_y + 3, 3, ": Pick from letter");
  }

  list->delay_update();

  ReadFile();
}

void AddressBook::Delete()
{
        DestroyChain();
        delete list;
}

void AddressBook::extrakeys(int key)
{
        switch (key) {
        case '\r':
        case '\n':
                if (!NoEnter)
                        SetLetterThings();
                break;
        case KEY_RIGHT:
        //case KEY2_RIGHT:
                Move(DOWN);
                Draw();
                break;
        case KEY_LEFT:
        //case KEY2_LEFT:
                Move(UP);
                Draw();
                break;
        case 'P':
                if (inletter)
                        AddressFromLetter();
        }
}

void AddressBook::SetLetterThings()
{
        if (highlighted)
                interface->letterwindow.set_Letter_Params(
                        &(highlighted->netmail_addr), highlighted->name);
}

void AddressBook::AddressFromLetter()
{
        net_address Address = interface->letterwindow.PickNetAddr();

        FILE *fd = fopen(bm.resourceObject->get(AddressFile), "at");
        fprintf(fd, "%s\n%s\n\n", bm.letterList->getFrom(), Address.text());
        fclose(fd);

        ReadFile();
}

void AddressBook::oneLine(int i)
{
        int z = position + i;
        Person *curr = (z < NumOfPersons) ? people[z] : 0;

        if (z == active)
                highlighted = curr;

        char *tmp = list->lineBuf;
        int x = curr ? sprintf(tmp, " %-31s %s", curr->name,
                curr->netmail_addr.text()) : 0;
        for (; x < list_max_x; x++)
                tmp[x] = ' ';
        tmp[x] = '\0';

        DrawOne(i, C_ADDRLIST);
}

int AddressBook::NumOfItems()
{
        return NumOfPersons;
}

void AddressBook::ReadFile()
{
        FILE *fd;
        Person head, *curr;
        char name[256], nmaddr[256], other[256];
        int end = 0;

        oldPos = oldActive = -100;
        DestroyChain();

        if ((fd = fopen(bm.resourceObject->get(AddressFile), "rt"))) {
                curr = &head;
                while (!end) {
                        do
                                if (!fgets(name, 255, fd))
                                        end = 1;
                        while (name[0] == '\n' && !end);
                        if (!fgets(nmaddr, 255, fd))
                                end = 1;
                        else {
                                curr->next = new Person;
                                curr = curr->next;
                                name[strlen(name) - 1] = '\0';
                                name[29] = '\0';
                                strcpy(curr->name, name);
                                curr->netmail_addr.init(nmaddr);
                                NumOfPersons++;
                        }
                        do
                                if (!fgets(other, 255, fd))
                                        end = 1;
                        while (other[0] != '\n' && !end);
                }
                fclose(fd);
        }

        if (NumOfPersons) {
                people = new Person *[NumOfPersons];
                curr = head.next;
                int c = 0;
                while (curr) {
                        people[c++] = curr;
                        curr = curr->next;
                }
        }

        if (active > NumOfPersons)
                active = position = 0;
        Draw();
}

void AddressBook::DestroyChain()
{
        while (NumOfPersons)
                delete people[--NumOfPersons];
        delete[] people;
}
