/*
 * blueMail offline mail reader
 * ANSI image/text viewer

 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>
 Copyright (c) 1998 Ingo Brueckl <ib@wupperonline.de>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#include "interfac.h"
#include "mysystem.h"
#include "isoconv.h"

AnsiLine::AnsiLine(int space, AnsiLine *parent)
{
        next = 0;
        prev = parent;
        if (space) {
                text = new chtype[space];
                for (int i = 0; i < (space - 1); i++)
                        text[i] = ' ' | C_MBACK;
                text[space - 1] = 0;
        } else
                text = 0;
        length = 0;
}

AnsiLine::~AnsiLine()
{
        delete[] text;
}

AnsiLine *AnsiLine::getnext(int space)
{
        if (!next)
                if (space)
                        next = new AnsiLine(space, this);
        return next;
}

AnsiLine *AnsiLine::getprev()
{
        return prev;
}

void AnsiWindow::DestroyChain()
{
        while (NumOfLines)
                delete linelist[--NumOfLines];
        delete[] linelist;
}

int AnsiWindow::getparm()
{
        char *parm;
        int value;

        if (escparm[0]) {
                for (parm = escparm; (*parm != ';') && *parm; parm++);

                if (*parm == ';') {                     // more params after
                        *parm++ = '\0';
                        if (parm == escparm + 1)        // empty param
                                value = 1;
                        else
                                value = atoi(escparm);
                        strcpy(escparm, parm);
                } else {                                // last parameter
                        value = atoi(escparm);
                        escparm[0] = '\0';
                }
        } else                                          // empty last param
                value = 1;

        return value;
}

void AnsiWindow::colreset()
{
        cfl = crv = cbr = 0;
        ccf = COLOR_WHITE;
        ccb = COLOR_BLACK;
}

void AnsiWindow::colorset()
{
        static const int colortable[8] = {COLOR_BLACK, COLOR_RED,
                COLOR_GREEN, COLOR_YELLOW, COLOR_BLUE, COLOR_MAGENTA,
                COLOR_CYAN, COLOR_WHITE};
        int tmp;

        while(escparm[0]) {
                tmp = getparm();
                switch (tmp) {
                case 0:                 // reset colors
                        colreset();
                        break;
                case 1:                 // bright
                        cbr = 1;
                        break;
                case 5:                 // flashing
                        cfl = 1;
                        break;
                case 7:                 // reverse
                        crv = 1;
                        break;
                default:
                        if ((tmp > 29) && (tmp < 38))           // foreground
                                ccf = colortable[tmp - 30];
                        else if ((tmp > 39) && (tmp < 48))      // background
                                ccb = colortable[tmp - 40];
                }
        }

        // Attribute set

        // Check bold and blinking:

        attrib = (cbr ? A_BOLD : 0) | (cfl ? A_BLINK : 0) |

        // If animating, check for color pair 0 (assumes COLOR_BLACK == 0),
        // and for remapped black-on-black:

                ((!anim || ((ccb | ccf) || !(oldcolorx | oldcolory))) ?

        // In PDCurses, the A_REVERSE attribute resets the colors, so
        // I avoid using it here:

#ifdef __PDCURSES__
                (crv ? COL(ccb, ccf) : COL(ccf, ccb)) : (crv ?
                COL(oldcolory, oldcolorx) : COL(oldcolorx, oldcolory)));
#else
                COL(ccf, ccb) : COL(oldcolorx, oldcolory)) |
                (crv ? A_REVERSE : 0);
#endif

        // If not animating, mark color pair as used:

        if (!anim)
#ifdef __PDCURSES__
                if (crv)
                        colorsused[(ccb << 3) + ccf] = 1;
                else
#endif
                        colorsused[(ccf << 3) + ccb] = 1;
}

const unsigned char *AnsiWindow::escfig(const unsigned char *escode)
{
        char a[2];

        a[0] = *escode;
        a[1] = '\0';

        switch (a[0]) {
        case 'A':                       // cursor up
                cpy -= getparm();
                if (cpy < 0)
                        cpy = 0;
                break;
        case 'B':                       // cursor down
                cpy += getparm();
                if (anim && (cpy > (LINES - 2)))
                        cpy = LINES - 2;
                break;
        case 'C':                       // cursor right
                cpx += getparm();
                if (cpx > (COLS - 1))
                        cpx = COLS - 1;
                break;
        case 'D':                       // cursor left
                cpx -= getparm();
                if (cpx < 0)
                        cpx = 0;
                break;
        case 'J':                       // clear screen
                if (getparm() == 2) {
                        if (anim) {
                                animtext->Clear(C_MBACK);
                                animtext->attrib(0);
                                posreset();
                        } else {
                                AnsiLine *tmp;
                                curr = head->getnext();
                                while (curr) {
                                        tmp = curr;
                                        curr = curr->getnext();
                                        delete tmp;
                                }
                                delete head;
                                ResetChain();
                        }
                }
                break;
        case 'H':                       // set cursor position
        case 'f':
                cpy = getparm() - 1;
                cpx = getparm() - 1;
                break;
        case 's':                       // save position
                spx = cpx;
                spy = cpy;
                break;
        case 'u':                       // restore position
                cpx = spx;
                cpy = spy;
                break;
        case 'm':                       // attribute set (colors, etc.)
                colorset();
                break;
        default:
                if (((a[0] >= '0') && (a[0] <= '9')) || (a[0] == ';')) {
                        strcat(escparm, a);
                        return escfig(++escode);
                }
        }
        return escode;
}

void AnsiWindow::posreset()
{
        cpx = cpy = lpy = spx = spy = 0;
}

void AnsiWindow::checkpos()
{
        if (cpy > lpy) {
                for (; lpy != cpy; lpy++)                // move down and
                        curr = curr->getnext(COLS + 1);  // allocate space
        } else
                if (cpy < lpy) {
                        for (; lpy != cpy; lpy--)        // move up
                                curr = curr->getprev();
                }
        if ((cpy + 1) > NumOfLines)
                NumOfLines = cpy + 1;
}

void AnsiWindow::update(unsigned char c)
{
        if (!ansiAbort) {
                chtype ouch = attrib | (isoconvert ? (unsigned char)
                                dos2isotab[c] : c);
                int limit = LINES - 2;

                if (anim) {
                        if (cpy > limit) {
                                animtext->wscroll(cpy - limit);
                                cpy = limit;
                        }
                        animtext->putch(cpy, cpx++, ouch);
                        animtext->update();     //!! necessary?
                        ansiAbort |= animtext->keypressed();
                } else {
                        checkpos();
                                curr->text[cpx++] = ouch;
                        if (cpx > (int) curr->length)
                                        curr->length = cpx;
                }
                if (cpx == COLS) {
                        cpx = 0;
                        cpy++;
                }
        }
}

void AnsiWindow::ResetChain()
{
        head = new AnsiLine();
        curr = head;
        posreset();
        curr = curr->getnext(COLS + 1);
        NumOfLines = 1;
}

void AnsiWindow::MakeChain(const unsigned char *message)
{
        ansiAbort = 0;
        if (!anim)
                ResetChain();
        attrib = C_MBACK;
        colreset();

        while (*message && !ansiAbort) {
                switch (*message) {
                case 10:
                        cpy++;
                case 13:
                        cpx = 0;
                case 7:
                        break;
                case '`':
                case 27:
                        if (*++message == '[') {
                                escparm[0] = '\0';
                                message = escfig(++message);
                        } else {
                                message--;
                                update('`');
                        }
                        break;
                default:
                        update(*message);
                }
                message++;
        }

        if (!anim && !ansiAbort) {
                linelist = new AnsiLine *[NumOfLines];
                curr = head->getnext();
                int i = 0;
                while (curr) {
                        linelist[i++] = curr;
                        curr = curr->getnext();
                }
                delete head;
        }
}

void AnsiWindow::statupdate(const char *intro)
{
        const char *mainstat = " ANSI View";
        char format[30], *tmp = new char[COLS + 1];

        statbar->cursor_on();
        sprintf(format, "%%s: %%-%d.%ds", COLS - 12, COLS - 12);
        sprintf(tmp, format, (intro ? intro : mainstat), title);
        statbar->putstring(0, 0, tmp);

        delete[] tmp;
        statbar->update();     //!!necessary?
        statbar->cursor_off();
}

void AnsiWindow::animate()
{
        animtext = new Win(LINES - 1, COLS, 0, 0, C_MBACK);
        animtext->attrib(0);
        animtext->cursor_on();

        posreset();
        colreset();
        anim = 1;

        statupdate(" Animating");

        MakeChain(source);

        if (!ansiAbort)
                statupdate("      Done");

        anim = 0;
        while (!ansiAbort && !animtext->keypressed());

        animtext->cursor_off();
        delete animtext;

        header->touch();     //!! necessary?
        text->touch();       //!! necessary?
        statupdate();
}

void AnsiWindow::oneLine(int i)
{
        text->putchnstr(i, 0, linelist[position + i]->text, COLS);
}

void AnsiWindow::DrawBody()
{
        int i, j;

        for (i = 0; i < y; i++)
                if ((position + i) < NumOfLines)
                        oneLine(i);
                else
                        for (j = 0; j < x; j++)
                                text->putch(i, j, ' ' | C_MBACK);
}

void AnsiWindow::set(const char *ansiSource, const char *winTitle)
{
        source = (const unsigned char *) ansiSource;
        position = 0;

        title = winTitle;
}

void AnsiWindow::MakeActive()
{
        int i, end, oldAbort;

        header = new Win(1, COLS, 0, 0, C_LBOTTLINE);
        text = new Win(LINES - 2, COLS, 1, 0, C_MBACK);
        statbar = new Win(1, COLS, LINES - 1, 0, C_LBOTTLINE);

        text->attrib(0);

        anim = 0;
        oldAbort = ansiAbort;

        char *tmp = new char[COLS + 1];
        sprintf(tmp, " " BM_TOPHEADER, BM_NAME, BM_MAJOR, BM_MINOR);
        for (i = strlen(tmp); i < COLS; i++)
                tmp[i] = ' ';
        tmp[i] = '\0';

        header->putstring(0, 0, tmp);
        header->delay_update();
        delete[] tmp;

        y = LINES - 2;
        x = COLS;

        for (i = 0; i < 64; i++)
                colorsused[i] = 0;

        colorsused[PAIR_NUMBER(C_LBOTTLINE)] = 1;  //don't remap stat bars

        oldcolorx = oldcolory = 0;

        init_pair(((COLOR_WHITE << 3) + COLOR_WHITE),
                COLOR_WHITE, COLOR_WHITE);

        MakeChain(source);

        // This deals with the unavailability of color pair 0:

        if (colorsused[0]) {    // assumes COLOR_BLACK == 0

                // Find an unused color pair for black-on-black:

                for (end = 0, i = 1; i < 64 && !end; i++)
                        if (!colorsused[i]) {
                                end = 1;
                                oldcolorx = i >> 3;
                                oldcolory = i & 7;
                                init_pair(i, COLOR_BLACK, COLOR_BLACK);
                        }

                // Remap all instances of color pair 0 to the new pair:

                if (end) {
                        for (int j = 0; j < NumOfLines; j++) {
                                curr = linelist[j];
                                for (i = 0; i < (int) curr->length; i++)
                                        if (!PAIR_NUMBER(curr->text[i])) {
                                                curr->text[i] &= ~A_COLOR;
                                                curr->text[i] |=
                                                  COL(oldcolorx, oldcolory);
                                        }
                        }
                }
        }

        DrawBody();
        text->delay_update();

        statupdate();

        ansiAbort = oldAbort;
}

void AnsiWindow::Delete()
{
        ansiAbort = 1;

        // Restore remapped color pairs:

        if (oldcolorx + oldcolory)
                init_pair(((oldcolorx << 3) + oldcolory),
                        oldcolorx, oldcolory);
        init_pair(((COLOR_WHITE << 3) + COLOR_WHITE),
                COLOR_BLACK, COLOR_BLACK);

        DestroyChain();
        delete header;
        delete text;
        delete statbar;
}

void AnsiWindow::KeyHandle(int key)
{
        switch (key) {
        case KEY_UP:
                if (position > 0) {
                        position--;
                        text->wscroll(-1);
                        oneLine(0);
                }
                break;
        case KEY_DOWN:
                if (position < NumOfLines - y) {
                        position++;
                        text->wscroll(1);
                        oneLine(y - 1);
                }
                break;
        case KEY_HOME:
                position = 0;
                DrawBody();
                break;
        case KEY_END:
                if (NumOfLines > y) {
                        position = NumOfLines - y;
                        DrawBody();
                }
                break;
        case KEY_PPAGE:
        //case KEY2_PPAGE:
                position -= ((y < position) ? y : position);
                DrawBody();
                break;
        case ' ':
        case KEY_NPAGE:
        //case KEY2_NPAGE:
                if (position < NumOfLines - y) {
                        position += y;
                        if (position > NumOfLines - y)
                                position = NumOfLines - y;
                        DrawBody();
                }
                break;
        case 'V':
                animate();
        }
        text->delay_update();
}
