/*
 * blueMail offline mail reader
 * area list

 Copyright (c) 1996 Kolossvary Tamas <thomas@vma.bme.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>
 Copyright (c) 1998 Ingo Brueckl <ib@wupperonline.de>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#include "interfac.h"
#include "isoconv.h"

// LittleAreaListWindow

LittleAreaListWindow::LittleAreaListWindow(char param_key)
{
        int dest;

        disp = 0;
        do
                bm.areaList->gotoArea(disp++);
        while (bm.areaList->isCollection());
        disp--;

        key = param_key;
        list_max_y = LINES - 10;
        list = new InfoWin(LINES - 6, 25, 2, (COLS - 25) / 2,
                C_LALBORDER, NULL, 0, 4, 3);
        dest = bm.letterList->getAreaID() - disp;
        while (active != dest)
                Move(DOWN);
        list->attrib(C_LALHEADER);
        list->putstring(1, 2, "Reply goes to area:");
        list->delay_update();
        Draw();
}

LittleAreaListWindow::~LittleAreaListWindow()
{
        bm.areaList->gotoArea(active + disp);
        interface->letterwindow.set_Letter_Params(active + disp, key);
        delete list;
}

int LittleAreaListWindow::NumOfItems()
{
        return bm.areaList->noOfAreas() - disp;
}

void LittleAreaListWindow::oneLine(int i)
{
        bm.areaList->gotoArea(position + i + disp);
        sprintf(list->lineBuf, " %-21.21s ", bm.areaList->getDescription());
        DrawOne(i, C_LALIST);
}

void LittleAreaListWindow::extrakeys(int key)
{
        if (key == 'C')
                interface->TransToggle();
}

// AreaListWindow

void AreaListWindow::Touch ()
{
  list->touch();
  if (!bm.resourceObject->isYes(SupprAreaLInfo)) info->touch();
}

void AreaListWindow::FirstUnread()
{
        int i;

        oldPos = oldActive = -100;
        position = active = 0;
        list_max_y = LINES - 15 +
                     (bm.resourceObject->isYes(SupprAreaLInfo) ? 4 : 0);
        for (i = 0; i < NumOfItems(); i++) {
                bm.areaList->gotoActive(i);
                if (!bm.areaList->getNoOfUnread())
                        Move(DOWN);
                else
                        break;
        }
        Draw();
}

int AreaListWindow::NumOfItems ()
{
  return bm.areaList->noOfActive();
}

void AreaListWindow::oneLine (int i)
{
  char *p = list->lineBuf;

  bm.areaList->gotoActive(position + i);

  if ((position + i == active) && !bm.resourceObject->isYes(SupprAreaLInfo))
  {
    sprintf(p, "%-20.20s", bm.areaList->getAreaType());
    info->putstring(2, 8, p);
    sprintf(p, format2, bm.areaList->getDescription());
    charconv_in(p);
    info->putstring(3, 20, p);
    info->delay_update();
  }
  p += sprintf(p, format, bm.areaList->getShortName(),
               bm.areaList->getDescription());

  if (bm.areaList->getNoOfLetters())
    p += sprintf(p, "  %5d  ", bm.areaList->getNoOfLetters());
  else p += sprintf(p, "      .  ");

  if (bm.areaList->getNoOfUnread())
    p += sprintf(p, "  %5d   ", bm.areaList->getNoOfUnread());
  else p += sprintf(p, "      .   ");

  if (mode)
    if (bm.areaList->getNoOfPersonal())
      sprintf(p, "   %5d   ", bm.areaList->getNoOfPersonal());
    else sprintf(p, "       .   ");

  DrawOne(i,
  ((bm.areaList->isReplyArea()
  ?
  C_ALREPLY | ((bm.areaList->getNoOfLetters() >0) ? A_BOLD : A_NORMAL)
  :
  ((bm.areaList->getNoOfUnread() > 0) ? C_ALISTUNRD : C_ALISTREAD))));
}

void AreaListWindow::Select ()
{
  bm.areaList->gotoActive(active);
}

void AreaListWindow::ResetActive ()
{
  active = bm.areaList->getActive();
}

void AreaListWindow::MakeActive()
{
  int padding;
  static char tmp[25];

  oldPos = oldActive = -100;
  mode = bm.driverList->hasPersonal();

  sprintf(tmp, "Message Areas (%d)", NumOfItems());

  list_max_y = LINES - 15 +
               (bm.resourceObject->isYes(SupprAreaLInfo) ? 4 : 0);

  list = new InfoWin(list_max_y + 3, COLS - 4, 2, 2, C_ALBORDER, tmp, C_ALTOPTEXT);

  list->attrib(C_ALHEADER);
  list->putstring(1, 3, "Area#  Description");
  int newloc = COLS - 21;
  if (mode) newloc -= 11;
  list->putstring(1, newloc, "Total   Unread");
  if (mode) list->putstring(1, COLS - 15, "Personal");

  padding = COLS - 34;
  if (mode) padding -= 11;
  sprintf(format, " %%6s  %%-%d.%ds", padding, padding);
  list->delay_update();

  if (!bm.resourceObject->isYes(SupprAreaLInfo))
  {
    info = new ShadowedWin(5, COLS - 4, LINES - 11, 2, C_ALBORDER,
                           "Info On The Specified Area", C_ALTOPTEXT);

    info->putch(0, 0, ACS_LTEE);
    info->putch(0, COLS - 5, ACS_RTEE);

    info->attrib(C_ALINFODESCR);
    info->putstring(1, 2, " BBS:                        Sysop: ");
    info->putstring(2, 2, "Type:                       Packet: ");
    info->putstring(3, 2, "Area description: ");

    info->attrib(C_ALINFOTEXT);
    sprintf(tmp, "%-20.20s", bm.resourceObject->get(BBSName));
    charconv_in(tmp);
    info->putstring(1, 8, tmp);

    sprintf(tmp, "%-20.20s", bm.resourceObject->get(SysOpName));
    charconv_in(tmp);
    info->putstring(1, 38, tmp);

    info->putstring(2, 38, bm.resourceObject->get(PacketName));

    padding = COLS - 26;
    sprintf(format2, "%%-%d.%ds", padding, padding);
  }

  Draw();
  Select();
}

void AreaListWindow::Delete ()
{
  delete list;
  if (!bm.resourceObject->isYes(SupprAreaLInfo)) delete info;
}

void AreaListWindow::PrevAvail()
{
        do {
                Move(UP);
                Select();
        } while (!bm.areaList->getNoOfLetters() &&
                 (bm.areaList->getActive() > 0));
        Draw();
}

void AreaListWindow::NextAvail()
{
        do {
                Move(DOWN);
                Select();
        } while (!bm.areaList->getNoOfLetters() &&
                 (bm.areaList->getActive() <
                  bm.areaList->noOfActive() - 1));
        Draw();
}

void AreaListWindow::extrakeys(int key)
{
        Select();
        switch (key) {
        case 'A':
                interface->newstate(address);
                break;
        case 'C':
                interface->TransToggle();
                break;
        case 'E':
                if (!bm.areaList->isCollection()) {
                        if (bm.areaList->isNetmail()) {
                                interface->newstate(address);
                                Select();
                        }
                        interface->letterwindow.set_Letter_Params(
                                bm.areaList->getAreaNo(), 'E');
                        interface->letterwindow.EnterLetter();
                        interface->changestate(arealist);
                }
                break;
        case KEY_LEFT:
        //case KEY2_LEFT:
                PrevAvail();
                break;
        case KEY_RIGHT:
        //case KEY2_RIGHT:
                NextAvail();
                break;
        case KEY_F(2):
        case '!':
                if (interface->WarningWindow(
        "This will overwrite any existing reply packet. Continue?") > 0)
                        interface->create_reply_packet();
                Touch();
                break;
        case 'L':
                bm.areaList->relist();
                ResetActive();
                interface->changestate(arealist);
        }
}
