/*
 * blueMail offline mail reader
 * help windows

 Copyright (c) 1996 Kolossvary Tamas <thomas@vma.bme.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>
 Copyright (c) 1998 Ingo Brueckl <ib@wupperonline.de>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#include "interfac.h"

void HelpWindow::newHelpMenu (const char **keys, const char **func)
{
  int y, z;
  int l, max_l = 0, max_m = 0, max_r = 0;

  menu = new Win(3, COLS - 2, LINES - 4, 1, C_HELPKEYS);

  for (z = -1, y = 0; y < 3; y++)
  {
    if ((l = slen(keys[++z]) + slen(func[z])) > max_l) max_l = l;
    if ((l = slen(keys[++z]) + slen(func[z])) > max_m) max_m = l;
    if ((l = slen(keys[++z]) + slen(func[z])) > max_r) max_r = l;
  }

  midpos = interface->midpos(2, max_l, max_m, max_r);
  endpos = interface->endpos(2, max_r);

  for (z = -1, y = 0; y < 3; y++)
  {
    if (keys[++z]) menu->putstring(y, 1, keys[z]);
    if (keys[++z]) menu->putstring(y, midpos, keys[z]);
    if (keys[++z]) menu->putstring(y, endpos, keys[z]);
  }

  menu->attrib(C_HELPDESCR);

  for (z = -1, y = 0; y < 3; y++)
  {
    if (func[++z]) menu->putstring(y, 1 + strlen(keys[z]), func[z]);
    if (func[++z]) menu->putstring(y, midpos + strlen(keys[z]), func[z]);
    if (func[++z]) menu->putstring(y, endpos + strlen(keys[z]), func[z]);
  }
}

void HelpWindow::h_packetlist ()
{
  static const char *keys[] =
  {
    "A", NULL, NULL,
    "K", NULL, NULL,
    NULL, NULL, "Q"
  },
  *func[] =
  {
    ": Addressbook", NULL, NULL,
    ": Kill packet", NULL, NULL,
    NULL, NULL, ": Quit " BM_NAME
  };

  newHelpMenu(keys, func);
  menu->delay_update();
}

void HelpWindow::h_arealist ()
{
  static const char *keys[] =
  {
    "A", NULL, NULL,
    "E", NULL, NULL,
    "L", NULL, "Q"
  },
  *func[] =
  {
    ": Addressbook", NULL, NULL,
    ": Enter letter in area", NULL, NULL,
    ": Long/short area list", NULL, ": Back to packet list"
  };

  newHelpMenu(keys, func);

  bm.areaList->gotoArea(REPLY_AREA);
  if (bm.areaList->getNoOfLetters() > 0)
  {
    midpos = interface->midpos(2, 23, 24, 22);   // length from strings above
    menu->attrib(C_HELPKEYS);
    menu->putstring(0, midpos, "F2, !");
    menu->attrib(C_HELPDESCR);
    menu->putstring(0, midpos + 5, ": Make reply packet");
  }
  menu->delay_update();
}

void HelpWindow::h_letterlist ()
{
  static const char *keys[] =
  {
    "A", "M", NULL,
    "E", "S", NULL,
    "L", "U", "Q"
  },
  *func[] =
  {
    ": Addressbook", ": Mark/unmark letter", NULL,
    ": Enter letter in area", ": Save messages", NULL,
    ": List all/unread", ": Unread/read toggle", ": Back to area list"

  },
  *repkeys[] =
  {
    "A", "S", NULL,
    "E", NULL, NULL,
    "K", NULL, "Q"
  },
  *repfunc[] =
  {
    ": Addressbook", ": Save all in area", NULL,
    ": Edit letter", NULL, NULL,
    ": Kill letter", NULL, ": Back to area list"
  };

  if (!bm.areaList->isReplyArea()) newHelpMenu(keys, func);
  else newHelpMenu(repkeys, repfunc);

  menu->delay_update();
}

void HelpWindow::h_letter ()
{
  int i, y = 1;

  static const char *common1[] =
  {
    "A: Addressbook",
    "C: toggle Character set translation",
    "",
    NULL
  },
  *common2[] =
  {
    "",
    "S: Save letter",
    "V: ANSI viewer",
    "Q: Back to letter list",
    "",
    "Space: Page through area",
    NULL
  },
  *regular[] =
  {
    "E: Enter letter",
    "M: Mark/unmark letter",
    "N: Netmail reply",
    "O: reply to Original sender",
    "R: Reply to letter",
    "T: Take tagline",
    "U: Unread/read toggle",
    NULL
  },
  *reply[] =
  {
    "E: Edit letter",
    "K: Kill letter",
    NULL
  };

  int l = bm.areaList->isReplyArea() ? 2 : 7;

  menu = new ShadowedWin(l + 11, 50, ((LINES - 11) - l) / 2,
                         COLS / 2 - 25, C_HELPBORDER);

  menu->attrib(C_HELPTEXT);

  for (i = 0; common1[i]; i++) menu->putstring(y++, 2, common1[i]);
  if (bm.areaList->isReplyArea())
    for (i = 0; reply[i]; i++) menu->putstring(y++, 2, reply[i]);
  else
    for (i = 0; regular[i]; i++) menu->putstring(y++, 2, regular[i]);
  for (i = 0; common2[i]; i++) menu->putstring(y++, 2, common2[i]);

  ((ShadowedWin *) menu)->delay_update();
}

void HelpWindow::MakeActive ()
{
  switch (interface->active())
  {
    case letter_help:
      h_letter();
      break;

    case letterlist:
      if (!bm.resourceObject->isYes(FullLetterList)) h_letterlist();
      break;

    case arealist:
      h_arealist();
      break;

    case packetlist:
      h_packetlist();
      break;

    default:
      ;
  }
}

void HelpWindow::Delete ()
{
  switch (interface->active())
  {
    case letter_help:
      delete (ShadowedWin *) menu;
      break;

    case letterlist:
      if (!bm.resourceObject->isYes(FullLetterList)) delete menu;
      break;

    case arealist:
    case packetlist:
      delete menu;
      break;

    default:
      ;
  }
}
