/*
 * blueMail offline mail reader
 * message list

 Copyright (c) 1996 Kolossvary Tamas <thomas@tvnet.hu>
 Copyright (c) 1997 John Zero <john@graphisoft.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>
 Copyright (c) 1998 Ingo Brueckl <ib@wupperonline.de>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#include "interfac.h"
#include "isoconv.h"
#include "mysystem.h"

const char *saveopts[] =
{
  "This",
  "Personal",
  "Marked",
  "Listed",
  "Quit"
};

void LetterListWindow::listSave ()
{
  int status = interface->WarningWindow("Save which?", saveopts, 5);

  if (status > 0)
    if (interface->letterwindow.Save(status) && (status == SAVE_THIS))
      Move(DOWN);

  interface->changestate(letterlist);
}

void LetterListWindow::NextUnread ()
{
  do
  {
    Move(DOWN);
    bm.letterList->gotoActive(active);
  }
  while (bm.letterList->getRead() && ((active + 1) < NumOfItems()));

  Draw();
}

void LetterListWindow::FirstUnread ()
{
  position = 0;
  active = -1;
  NextUnread();
}

void LetterListWindow::PrevUnread ()
{
  do
  {
    Move(UP);
    bm.letterList->gotoActive(active);
  }
  while (bm.letterList->getRead() && (active > 0));

  Draw();
}

int LetterListWindow::NumOfItems ()
{
  return bm.letterList->noOfActive();
}

void LetterListWindow::oneLine (int i)
{
  char c;
  chtype c_ll;
  const char *l, *a, *f;

  bm.letterList->gotoActive(position + i);

  const XTI_REC *st = bm.letterList->getStatus();

  char *p = list->lineBuf;
  p += sprintf(p, format, st->marks ? 'M' : ' ',
                          (st->flags & XTI_HAS_REPLIED) ? '~' : ' ',
                          (st->flags & XTI_HAS_READ) ? '*' : ' ',
               bm.letterList->getMsgNum(), bm.letterList->getFrom(),
               bm.letterList->getTo(), bm.letterList->getSubject());

  if (bm.areaList->isCollection())
  {
    const char *origArea = bm.areaList->
      getDescription(bm.letterList->getAreaID());
    if (origArea)
    {
      if (bm.resourceObject->isYes(FullLetterList))
        for (unsigned int i = 0; i < 25; i++)
        {
          if (i < strlen(origArea)) c = origArea[i];
          else c = ' ';
          p[i - 74] = c;
        }
      else sprintf(p - 15, " %-13.13s ", origArea);
    }
  }

  l = bm.resourceObject->get(LoginName);
  a = bm.resourceObject->get(AliasName);
  f = bm.letterList->getFrom();

  c_ll = C_LLISTUNRD;
  if (strcasecmp(l, f) == 0 || strcasecmp(a, f) == 0) c_ll = C_LLFROMUSER;
  if (bm.letterList->isPersonal()) c_ll = C_LLTOUSER;
  if (st->flags & XTI_HAS_READ) c_ll = C_LLISTREAD;

  DrawOne(i, c_ll);
}

void LetterListWindow::MakeActive ()
{
  static char topline[72];
  int tot, maxFromLen, maxToLen, maxSubjLen;
  int xmore = 0, ymore = 0;

  if (bm.resourceObject->isYes(FullLetterList))
  {
    xmore = 2;
    ymore = 8;
  }

  oldPos = oldActive = -100;
  tot = COLS - 19 + xmore + xmore;
  maxSubjLen = tot / 2 - xmore;
  tot -= maxSubjLen;
  maxToLen = tot / 2;
  maxFromLen = tot - maxToLen;

  sprintf(format, "%%c%%c%%c%%6d  %%-%d.%ds %%-%d.%ds %%-%d.%ds",
          maxFromLen, maxFromLen, maxToLen, maxToLen, maxSubjLen, maxSubjLen);

  interface->areas.Select();

  sprintf(topline, "Letters in %.60s", bm.areaList->getDescription());
  charconv_in(topline);

  list_max_y = (NumOfItems() < LINES - 11 + ymore) ? NumOfItems()
                                                   : LINES - 11 + ymore;
  list = new InfoWin(ymore ? LINES : list_max_y + 3,
                     COLS - 4 + xmore + xmore, 2 - xmore, 2 - xmore,
                     C_LLBORDER, topline, C_LLAREA);

  list->attrib(C_LLTOPTEXT);
  list->putstring(0, 3, "Letters in ");

  sprintf(topline, "   Msg#  %%-%d.%ds %%-%d.%ds %%-%d.%ds",
          maxFromLen, maxFromLen, maxToLen, maxToLen, maxSubjLen, maxSubjLen);
  list->attrib(C_LLHEADER);
  sprintf(list->lineBuf, topline, "From", "To", "Subject");
  list->putstring(1, 3, list->lineBuf);

  if (bm.areaList->isCollection())
    if (ymore) list->putstring(1, 5, "Area       ");
    else list->putstring(1, COLS - 19, "Area");

  list->delay_update();
  Draw();
  Select();
}

void LetterListWindow::Select()
{
        bm.letterList->gotoActive(active);
}

void LetterListWindow::ResetActive()
{
        active = bm.letterList->getActive();
}

void LetterListWindow::Delete()
{
        delete list;
}

void LetterListWindow::StatToggle (tBYTE flag, tBYTE mark)
{
  XTI_REC newstat;

  const XTI_REC *stat = bm.letterList->getStatus();
  newstat.flags = stat->flags ^ flag;
  newstat.marks = stat->marks ^ mark;
  bm.letterList->setStatus(newstat.flags, newstat.marks);
}

void LetterListWindow::extrakeys(int key)
{
  tBYTE f, m;

        Select();
        switch (key) {
        case 'A':
                interface->newstate(address);
                break;
        case 'C':
                interface->TransToggle();
                break;
        case KEY_LEFT:
        //case KEY2_LEFT:
                PrevUnread();
                break;
        case KEY_RIGHT:
        //case KEY2_RIGHT:
                NextUnread();
                break;
        case 'U':
        case 'M':       // Toggle read/unread and marked from letterlist
                if (key == 'U')
                {
                  f = XTI_HAS_READ;
                  m = 0;
                }
                else
                {
                  f = 0;
                  m = XTI_MARK_SAVE | XTI_MARK_REPLY |    // as blueMail has
                      XTI_MARK_PRINT | XTI_MARK_DELETE;   // only one mark,
                }                                         // set them all
                StatToggle(f, m);
                interface->setAnyRead();
                Move(DOWN);
                Draw();
                break;
        case 'E':
                if (bm.areaList->isReplyArea())
                        interface->letterwindow.KeyHandle('E');
                else
                    if (!bm.areaList->isCollection()) {
                        if (bm.areaList->isNetmail())
                                interface->newstate(address);
                        interface->letterwindow.set_Letter_Params(
                                bm.areaList->getAreaNo(), 'E');
                        interface->letterwindow.EnterLetter();
                    }
                interface->changestate(letterlist);
                break;
        case 'K':
                if (bm.areaList->isReplyArea())
                        interface->kill_letter();
                break;
        case 'L':
                bm.letterList->relist();
                ResetActive();
                interface->changestate(letterlist);
                break;
        case 'S':
                listSave();
        }
}
