/*
 * blueMail offline mail reader
 * message display and editing

 Copyright (c) 1996 Kolossvary Tamas <thomas@tvnet.hu>
 Copyright (c) 1997 John Zero <john@graphisoft.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>
 Copyright (c) 1998 Ingo Brueckl <ib@wupperonline.de>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#include "interfac.h"
#include "isoconv.h"
#include "mysystem.h"

#include <sys/utsname.h>

Line::Line(int size)
{
        next = NULL;
        text = (size ? new char[size] : 0);
}

Line::~Line()
{
        delete[] text;
}

LetterWindow::LetterWindow()
{
        linelist = NULL;
        NM.zone = 0;
        NumOfLines = 0;
        To[0] = tagline1[0] = '\0';
}

net_address LetterWindow::PickNetAddr()
{
        Line *line;
        net_address result;
        int i;

        line = linelist[0];
        for (i = 0; i < NumOfLines; i++)
                if (!strncmp(" * Origin:", linelist[i]->text, 10))
                        break;

        if (i != NumOfLines) {
                line = linelist[i];
                i = line->length;
                while (((line->text[i - 1]) != '(') && i > 0)
                        i--;
                //we have the opening bracket

                while ((i < (int) line->length) &&
                        ((line->text[i] < '0') || (line->text[i] > '9')))
                                i++;
                //we have the begining of the address

                result.init(&line->text[i]);
        } else
                result = *(bm.letterList->getNetAddr());

        return result;
}

void LetterWindow::Touch ()
{
  if (interface->active() == letter)
  {
    header->touch();
    text->touch();
    statbar->cursor_on();     // to get it out of the way
    statbar->touch();
    statbar->cursor_off();
  }
}

void LetterWindow::DestroyChain()
{
        if (linelist) {
                while (NumOfLines)
                        delete linelist[--NumOfLines];
                delete[] linelist;
                linelist = NULL;
        }
        letter_in_chain = -1;
}

void LetterWindow::MakeChain(int columns)
{
        Line head(0), *curr;
        char *message, *j, *k, *begin;
        int c, end = 0;

        DestroyChain();
        message = (char *) bm.letterList->getBody();
        charconv_in(message);
        letter_in_chain = bm.letterList->getCurrent();
        curr = &head;

        j = message;
        while (!end) {
                curr->next = new Line(columns + 1);
                curr = curr->next;

                NumOfLines++;

                begin = NULL;
                k = curr->text;
                c = 0;

                while ((*j != 10) && !end && (c < columns)) {
                        if (*j == ' ')
                                begin = NULL;
                        else
                                if (!begin)
                                        begin = j;
                        if (*j) {
                                *k++ = *j++;
                                c++;
                        } else
                                end = 1;
                }
                if ((c == columns) && begin && ((j - begin) < columns)) {
                        c -= j - begin;
                        j = begin;
                }
                if (*j == 10)
                        j++;

                curr->text[c] = '\0';
                curr->length = c;
        }

        linelist = new Line *[NumOfLines];
        curr = head.next;
        c = 0;
        while (curr) {
                linelist[c++] = curr;
                curr = curr->next;
        }
}

void LetterWindow::StatToggle (tBYTE flag, tBYTE mark)
{
  XTI_REC newstat;

  const XTI_REC *stat = bm.letterList->getStatus();
  newstat.flags = stat->flags ^ flag;
  newstat.marks = stat->marks ^ mark;
  bm.letterList->setStatus(newstat.flags, newstat.marks);
  DrawHeader();
}

void LetterWindow::Draw(bool redo)
{
       if (redo) {
               position = 0;
               tagline1[0] = '\0';
               bm.letterList->setRead();       // nem ide kene? de.
       }
        DrawHeader();
        DrawBody();
}

void LetterWindow::netAdd(char *tmp)
{
        net_address na = *(bm.letterList->getNetAddr());
        if (na.zone) {
                strcat(tmp, " @ ");
                strcat(tmp, na.text());
        }
}

#define flagattr(x) header->attrib((x) ? C_LHFLAGSHI : C_LHFLAGS)

void LetterWindow::DrawHeader ()
{
  int ymove = 0;
  static char tmp[80];
  const XTI_REC *stat = bm.letterList->getStatus();

  if (bm.resourceObject->isYes(SupprLineCounter)) ymove = 1;

  header->Clear(C_LHTEXT);

  header->attrib(C_LHBORDER);
  header->Border();

  header->attrib(C_LHTEXT);
  header->putstring(0, 1, " Msg#:");
  header->putstring(1, 2, "From:");
  header->putstring(2, 2, "  To:");
  header->putstring(3, 2, "Subj:");

  header->putstring(1, COLS - 33, "Line:");
  header->putstring(0 + ymove, COLS - 34, " Date:");
  header->putstring(2, COLS - 33, "Stat:");

  flagattr(bm.letterList->getPrivate());
  header->putstring(2, COLS - 27, "Pvt");
  flagattr(stat->flags & XTI_HAS_READ);
  header->putstring(2, COLS - 23, "Read");
  flagattr(stat->flags & XTI_HAS_REPLIED);
  header->putstring(2, COLS - 18, "Replied");
  flagattr(stat->marks);
  header->putstring(2, COLS - 10, "Marked");

  header->attrib(C_LHMSGNUM);
  sprintf(tmp, " %d of %d (%d) ", bm.letterList->getCurrent() + 1,
          bm.areaList->getNoOfLetters(), bm.letterList->getMsgNum());
  header->putstring(0, 7, tmp);

  header->attrib(C_LHFROM);
  strcpy(tmp, bm.letterList->getFrom());
  charconv_in(tmp);

  if (bm.areaList->isNetmail()) netAdd(tmp);

  header->putstring(1, 8, tmp);

  header->attrib(C_LHTO);
  strcpy(tmp, bm.letterList->getTo());
  charconv_in(tmp);

  if (bm.areaList->isReplyArea()) netAdd(tmp);

  header->putstring(2, 8, tmp);

  header->attrib(C_LHSUBJ);
  sprintf(tmp, "%-71.71s", bm.letterList->getSubject());
  charconv_in(tmp);
  header->putstring(3, 8, tmp);

  for (int i = strlen(tmp) + 8; i < COLS - 1; i++) header->putch(3, i, ' ');

  header->attrib(C_LHDATE);
  sprintf(tmp, " %s ", bm.letterList->getDate());
  charconv_in(tmp);
  header->putstring(0 + ymove, COLS - 28, tmp);

  lineCount();
}

void LetterWindow::lineCount ()
{
  char tmp[80];

  if (!bm.resourceObject->isYes(SupprLineCounter))
  {
    header->attrib(C_LHMSGNUM);
    sprintf(tmp, "%5d/%-6d", position + 1, NumOfLines);
    header->putstring(1, COLS-27, tmp);
  }

  header->delay_update();
}

void LetterWindow::oneLine (int i)
{
  char test, *t, *r, *currtext;
  int j, length, z = position + i;

  if (z < NumOfLines)
  {
    currtext = linelist[z]->text;
    length = (int) linelist[z]->length;

    text->attrib(C_LTEXT);

    for (j = 0; j < 5; j++)     // quoting
      if (currtext[j] == '>')
      {
        text->attrib(C_LQTEXT);
        break;
      }
    if ((currtext[0] == ':' || currtext[0] == '|') && currtext[1] == ' ')
      text->attrib(C_LQTEXT);

    test = currtext[0];
    if ((currtext[1] == test) && (currtext[2] == test) &&
       ((currtext[3] == ' ') || length == 3))
      switch (test)
      {
        case '.':     // tagline
          text->attrib(C_LTAGLINE);
          t = &currtext[4];
          r = tagline1;
          if (!*r)
          {
            for (j = 0; (*t) && (*t != '\r') && (*t != '\n') && j < 79;
                 t++, r++, j++) *r = *t;
            *r = 0;
          }
          break;
        case '-':     // tearline
        case '~':
          text->attrib(C_LTEARLINE);
      }
    else if (!strncmp(currtext, " * Origin: ", 11)) text->attrib(C_LORIGIN);

    text->putstring(i, 0, currtext);
  }
  else length = 0;

  for (j = length; j < COLS; j++) text->putch(i, j, ' ');
}

void LetterWindow::DrawBody()
{
        if (letter_in_chain != bm.letterList->getCurrent())
                MakeChain(COLS);

        lineCount();

        for (int i = 0; i < y; i++)
                oneLine(i);
        text->delay_update();
}

void LetterWindow::MakeActive (bool redo)
{
  DestroyChain();

  y = LINES - 6;

  header = new Win(5, COLS, 0, 0, C_LHTEXT);
  text = new Win(y, COLS, 5, 0, C_LTEXT);
  statbar = new Win(1, COLS, LINES - 1, 0, C_LBOTTLINE);

  statbar->cursor_on();

  char *tmp = new char[COLS + 1];
  static const char *helpmsg = "F1 or ? - Help ";
  char format[20];
  int maxw = COLS - 16;

  sprintf(format, " %%-%d.%ds%%s", maxw, maxw);
  sprintf(tmp, format, bm.areaList->getDescription(), helpmsg);
  charconv_in(tmp);

  statbar->putstring(0, 0, tmp);
  delete[] tmp;

  statbar->delay_update();
  statbar->cursor_off();

  // linelist = 0;
  // NumOfLines = 0;
  Draw(redo);
}

void LetterWindow::Next()
{
        if (bm.letterList->getActive() < (bm.letterList->noOfActive() - 1)) {
                interface->letters.Move(DOWN);
                bm.letterList->gotoActive(bm.letterList->getActive() + 1);
                Draw(true);
        } else {
                interface->back();
                interface->back();
                interface->areas.KeyHandle(KEY_RIGHT);
        }
}

void LetterWindow::Previous()
{
        if (bm.letterList->getActive() > 0) {
                interface->letters.Move(UP);
                bm.letterList->gotoActive(bm.letterList->getActive() - 1);
                Draw(true);
        } else {
                interface->back();
                interface->back();
                interface->areas.Move(UP);
        }
}

void LetterWindow::Move(int dir)
{
        switch (dir) {
        case KEY_UP:
                if (position > 0) {
                        position--;
                        text->wscroll(-1);
                        oneLine(0);
                        text->delay_update();
                        lineCount();
                }
                break;
        case KEY_DOWN:
                if (position < NumOfLines - y) {
                        position++;
                        lineCount();
                        text->wscroll(1);
                        oneLine(y - 1);
                        text->delay_update();
                }
                break;
        case KEY_HOME:
                position = 0;
                DrawBody();
                break;
        case KEY_END:
                if (NumOfLines > y) {
                        position = NumOfLines - y;
                        DrawBody();
                }
                break;
        case KEY_PPAGE:
        //case KEY2_PPAGE:
                position -= ((y < position) ? y : position);
                DrawBody();
                break;
        case KEY_NPAGE:
        //case KEY2_NPAGE:
                if (position < NumOfLines - y) {
                        position += y;
                        if (position > NumOfLines - y)
                                position = NumOfLines - y;
                        DrawBody();
                }
                break;
        }
}

void LetterWindow::NextDown()
{
        position += y;
        if (position >= NumOfLines)
                Next();
        else
                DrawBody();
}

void LetterWindow::Delete ()
{
  DestroyChain();
  delete header;
  delete text;
  delete statbar;
}

bool LetterWindow::Save (int stype)
{
  FILE *fd;
  static char savefile[MYMAXPATH];
  bool written = false;
  const XTI_REC *st;
  char tmp[26];

  ShadowedWin question(4, 60, (LINES >> 1) - 2, 10, C_SAVEBOXBORDER);
  question.attrib(C_SAVEBOXHEADER);
  question.putstring(1, 2, "Append to file:");
  question.putstring(2, 2, "<");
  question.putstring(2, 57, ">");
  question.delay_update();

  if (question.getstring(2, 3, savefile, 54, C_SAVEBOXINP, C_SAVEBOXINP))
  {
    mychdir(bm.resourceObject->get(SaveDir));
    if ((fd = fopen(savefile, "at")))
    {
      switch (stype)
      {
        case SAVE_LISTED:
          for (int i = 0; i < bm.letterList->noOfActive(); i++)
          {
            bm.letterList->gotoActive(i);
            write_to_file(fd);
            written = true;
          }
          break;

        case SAVE_MARKED:
        case SAVE_PERSONAL:
          for (int i = 0; i < bm.areaList->getNoOfLetters(); i++)
          {
            bm.letterList->gotoLetter(i);
            st = bm.letterList->getStatus();
            if (((stype == SAVE_MARKED) && st->marks) ||
                ((stype == SAVE_PERSONAL) && bm.letterList->isPersonal()))
            {
              write_to_file(fd);
              written = true;
            }
          }
          break;

        case SAVE_THIS:
          write_to_file(fd);
          written = true;
          break;
      }
      fclose(fd);
      if (stype == SAVE_THIS) MakeChain(COLS);     // ???
      if (!written)
      {
        sprintf(tmp, "No %.8s message found", saveopts[SAVE_MAX - stype]);
        tmp[3] = tolower(tmp[3]);
        interface->nonFatalError(tmp);
      }
    }
    return true;
  }
  else return false;
}

void LetterWindow::set_Letter_Params(net_address *nm, const char *to)
{
        NM = *nm;
        strncpy(To, to ? to : "", 29);
}

void LetterWindow::set_Letter_Params(int area, char param_key)
{
        key = param_key;
        if (key == 'N') {
                int i = bm.areaList->findNetmail();
                replyto_area = (i != -1) ? i : area;
        } else
                replyto_area = area;
}

void LetterWindow::QuoteText(const char *reply_filename)
{
        FILE *reply;
        char TMP[81], mg[4];
        int end, i;

        const char *TMP2 = bm.letterList->getFrom();

        sprintf(TMP, "-=> %.30s wrote to %.30s <=-\n\n",
                TMP2, bm.letterList->getTo());
        charconv_in(TMP);

        strncpy(mg, TMP2, 2);
        mg[2] = '\0';
        mg[3] = '\0';

        i = 1;
        for (int j = 1; j < 3; j++) {
                end = 0;
                while (TMP2[i] && !end) {
                        if ((TMP2[i - 1] == ' ') && (TMP2[i] != ' ')) {
                                mg[j] = TMP2[i];
                                if (j == 1)
                                        end = 1;
                        }
                        i++;
                }
        }
        charconv_in(mg);

        MakeChain(75);

        if ((reply = fopen(reply_filename, "wt"))) {
                fputs(TMP, reply);
                for (i = 0; i < NumOfLines; i++)
                        fprintf(reply, " %s> %s\n", mg, linelist[i]->text);
                fclose(reply);
        }

        MakeChain(COLS);
}

int LetterWindow::HeaderLine(ShadowedWin *X, char *Y, int limit, int pos,
                       chtype inp_color, chtype norm_color)
{
        charconv_in(Y);
        int getit = X->getstring(pos, 8, Y, limit, inp_color, norm_color);
        charconv_out(Y);
        return getit;
}

int LetterWindow::EnterHeader(char *FROM, char *TO, char *SUBJ, int *privat)
{
  static const char *noyes[] = {"No", "Yes"};

        ShadowedWin *rep_header;
        char NETADD[25];
        int result, end = 0, current = 0, maxitems = 3, r = 0;

        if (bm.areaList->isNetmail() || NM.zone) {
                strcpy(NETADD, NM.text());
                maxitems++;
        }

        rep_header = new ShadowedWin(maxitems + 2, COLS - 2, (LINES / 2) - 3,
                1, C_REPBOXBORDER);

        rep_header->attrib(C_REPBOXDESCR);
        rep_header->putstring(1, 2, "From:");
        rep_header->putstring(2, 2, "  To:");
        if (maxitems == 4)
                rep_header->putstring(3, 2, "Addr:");
        rep_header->putstring(maxitems, 2, "Subj:");
        rep_header->delay_update();

        do {
                result = HeaderLine(rep_header, (current == (maxitems - 1)) ?
                        SUBJ : (((current == 2) && (maxitems == 4)) ?
                        NETADD : (current ? TO : FROM)),
                        (current == (maxitems - 1)) ? 69 : 25,
                        current + 1, C_REPBOXINP, C_REPBOXTEXT);

                switch (result) {
                case 0:
                        end = 1;
                        break;
                case 1:
                        current++;
                        if (current == maxitems)
                                end = 1;
                        break;
                case 2:
                        if (current > 0)
                                current--;
                        break;
                case 3:
                        if (current < maxitems - 1)
                                current++;
                }
        } while (!end);

        if (result) {
                if (bm.areaList->isNetmail() || NM.zone)
                        NM.init(NETADD);
                if (( r = interface->WarningWindow("Make letter private?",
                                                   *privat ? 0 : noyes)) > 0)
                  if (!r) *privat = !*privat;
        }

        delete rep_header;

        return (r == -1) ? 0 : result;
}

void LetterWindow::editletter(const char *reply_filename)
{
        char command[256];

        sprintf(command, "%s %s", bm.resourceObject->get(editor),
                canonize(reply_filename));
        mysystem(command);
}

long LetterWindow::reconvert(const char *reply_filename)
{
        FILE *reply;

        reply = fopen(reply_filename, "rt");
        fseek(reply, 0, SEEK_END);
        long replen = ftell(reply);
        rewind(reply);

        char *body = new char[replen + 1];
        replen = (long) fread(body, 1, replen, reply);
        fclose(reply);

        body[replen] = '\0';
        charconv_out(body);
        while (body[replen - 1] == '\n')
                replen--;

        reply = fopen(reply_filename, "wt");
        fwrite(body, 1, replen, reply);
        fclose(reply);

        delete[] body;
        return replen;
}

void LetterWindow::EnterLetter()
{
        FILE *reply;
        char reply_filename[256], FROM[26], TO[26], SUBJ[78];
        long replen;
        int privat, replyto_num;

        bm.areaList->gotoArea(replyto_area);

        // HEADER

        int usealias = bm.areaList->getUseAlias();
        if (usealias) {
                sprintf(FROM, "%.25s", bm.resourceObject->get(AliasName));
                if (!FROM[0])
                        usealias = 0;
        }
        if (!usealias)
                sprintf(FROM, "%.25s", bm.resourceObject->get(LoginName));

        if (key == 'E') {
                strcpy(TO, (To[0] ? To : "All"));
                SUBJ[0] = '\0'; //we don't have subject yet
        } else {
                sprintf(TO, "%.25s", (key == 'O') ? bm.letterList->getTo() :
                        bm.letterList->getFrom());
                sprintf(SUBJ, "Re: %.65s", bm.letterList->getSubject());
        }

        if ((key == 'N') || bm.areaList->isNetmail())
                privat = 1;
        else
                privat = (key == 'E') ? 0 : bm.letterList->getPrivate();

        interface->areas.Select();

        if (!EnterHeader(FROM, TO, SUBJ, &privat))
        {
          Touch();
          return;
        }

        replyto_num = (key == 'E') ? 0 : bm.letterList->getMsgNum();

        // BODY

        mytmpnam(reply_filename);

        // Quote the old text, mark original as replied
        if (key != 'E') {
                QuoteText(reply_filename);
                const XTI_REC *stat = bm.letterList->getStatus();
                bm.letterList->setStatus(stat->flags | XTI_HAS_REPLIED, stat->marks);
        }
        DrawHeader();

        // edit the reply
        editletter(reply_filename);
        // some editors destroy the background
        Touch();

        reply = fopen(reply_filename, "at");

        // signature
        const char *sg = bm.resourceObject->get(sigFile);
        if (sg) {
                FILE *s = fopen(sg, "rt");
                if (s) {
                        char bufx[81];

                        fputc('\n', reply);
                        while (fgets(bufx, 79, s))
                                fputs(bufx, reply);
                        fclose(s);
                }
        }

        // Tagline
        tagline1[0] = '\0';

        interface->newstate(tagwin);
        interface->KeyHandle();

        if (tagline1[0])
                fprintf(reply, "\n... %s\n", tagline1);

        // Tearline (not for Blue Wave -- it does its own)
        if (bm.driverList->useTearline()) {
                struct utsname buf;
                uname(&buf);
                if (!tagline1[0])
                        fputc('\n', reply);
                fprintf(reply, "--- %s/%s " BM_TOP_V "\n", BM_NAME, buf.sysname,
                        BM_MAJOR, BM_MINOR);
        }

        fclose(reply);

        tagline1[0] = '\0';

        // Reconvert the text
        replen = reconvert(reply_filename);

        bm.areaList->enterLetter(replyto_area, FROM, TO, SUBJ, replyto_num,
                privat, &NM, reply_filename, (int) replen);

        NM.zone = 0;
        To[0] = '\0';

        interface->areas.Select();
        interface->setUnsaved();
}

void LetterWindow::EditLetter()
{
        FILE *reply;
        char reply_filename[256], FROM[26], TO[26], SUBJ[78], *body;
        long siz;
        int privat, replyto_num, replyto_area;

        strcpy(FROM, bm.letterList->getFrom());
        strcpy(TO, bm.letterList->getTo());
        strcpy(SUBJ, bm.letterList->getSubject());
        privat = bm.letterList->getPrivate();

        NM = *(bm.letterList->getNetAddr());

        if (!EnterHeader(FROM, TO, SUBJ, &privat))
        {
          Touch();
          return;
        }

        DestroyChain();         // current letter's chain reset

        replyto_num = bm.letterList->getReplyTo();
        replyto_area = bm.letterList->getAreaID();

        mytmpnam(reply_filename);

        body = (char *) bm.letterList->getBody();
        charconv_in(body);
        reply = fopen(reply_filename, "wt");
        fwrite(body, strlen(body), 1, reply);
        fclose(reply);
        body = NULL;            // it will be auto-dealloc'd by next getBody

        editletter(reply_filename);
        siz = reconvert(reply_filename);

        bm.areaList->killLetter(bm.letterList->getMsgNum());
        bm.areaList->enterLetter(replyto_area, FROM, TO, SUBJ, replyto_num,
                privat, &NM, reply_filename, (int) siz);

        bm.letterList->rrefresh();
        interface->areas.Select();
        if (interface->active() == letter)
                interface->back();

        NM.zone = 0;

        interface->setUnsaved();
}

void LetterWindow::set_Tagline(const char *tl)
{
        strncpy(tagline1, tl, 76);
}

void LetterWindow::GetTagline()
{
        interface->taglines.EnterTagline(tagline1);
        Touch();
}

void LetterWindow::write_to_file(FILE *fd)
{
        char Header[400];
        int j;

        // write header to file
        for (j = 0; j < 80; j++)
                fputc('=', fd);

        sprintf(Header,
                "\n System: %.71s\n   Area: %.71s\n   Date: %.25s\n"
                "   From: %.45s\n     To: %.45s\n   Subj: %.71s\n",
                bm.resourceObject->get(BBSName),
                bm.areaList->getDescription(), bm.letterList->getDate(),
                bm.letterList->getFrom(), bm.letterList->getTo(),
                bm.letterList->getSubject());

        charconv_in(Header);
        fputs(Header, fd);

        for (j = 0; j < 80; j++)
                fputc('-', fd);
        fputc('\n', fd);

        // write chain to file
        MakeChain(80);
        for (j = 0; j < NumOfLines; j++)
                fprintf(fd, "%s\n", linelist[j]->text);
        fputc('\n', fd);
}

void LetterWindow::KeyHandle(int key)
{
  tBYTE f, m;

        switch (key) {
        case 'A':
                interface->newstate(address);
                break;
        case 'C':
                interface->TransToggle();
                break;
        case 'S':
                Save(SAVE_THIS);
                Touch();
                break;
        case '?':
        case KEY_F(1):
                interface->changestate(letter_help);
                break;
        case 'V':
                interface->ansiLoop(bm.letterList->getBody(),
                        bm.letterList->getSubject());
                break;
        case KEY_RIGHT:
        //case KEY2_RIGHT:
                Next();
                break;
        case KEY_LEFT:
        //case KEY2_LEFT:
                Previous();
                break;
        case ' ':
                NextDown();
                break;
        default:
            if (bm.areaList->isReplyArea()) {
                switch(key) {
                case 'E':
                        EditLetter();
                        break;
                case 'K':
                        interface->kill_letter();
                        break;
                default:
                        Move(key);
                }
            } else {
                switch (key) {
                case 'M':
                case 'U':
                  if (key == 'M')
                  {
                    f = 0;
                    m = XTI_MARK_SAVE | XTI_MARK_REPLY |    // as blueMail has
                        XTI_MARK_PRINT | XTI_MARK_DELETE;   // only one mark,
                  }                                         // set them all
                  else
                  {
                    f = XTI_HAS_READ;
                    m = 0;
                  }
                        StatToggle(f, m);
                        break;
                case 'R':
                case 'O':
                case 'E':
                        interface->changestate(littlearealist);
                        break;
                case 'N':
                        if (bm.areaList->findNetmail() != -1) {
                            net_address nm = PickNetAddr();
                            if (nm.zone) {
                                set_Letter_Params(
                                        bm.areaList->getAreaNo(), 'N');
                                set_Letter_Params(&nm, NULL);
                                EnterLetter();
                            }
                        }
                        break;
                case 'T':
                        GetTagline();
                        break;
                default:
                        Move(key);
                }
            }
        }
}
