/*
 * blueMail offline mail reader
 * packet list window, vanity plate

 Copyright (c) 1996 Kolossvary Tamas <thomas@vma.bme.hu>
 Copyright (c) 1997 John Zero <john@graphisoft.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>
 Copyright (c) 1998 Ingo Brueckl <ib@wupperonline.de>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#include "interfac.h"
#include "mysystem.h"

void Welcome::MakeActive()
{
        window = new Win(6, 56, 2, COLS / 2 - 28, C_MBACK);
        window->attrib(C_WELCBORDER);
        window->Border();
        window->attrib(C_WELCHEADER);
        window->putstring(1, 8,
                "Welcome to " BM_NAME " " BM_IS_WHAT "!");
        window->attrib(C_WELCTEXT);
        window->putstring(3, 2,
                "Written by Ingo Brueckl, William McBrine, John Zero,");
        window->putstring(4, 2,
                "Kolossvary Tamas, Toth Istvan, and others.");
        //window->touch();
        window->delay_update();
}

void Welcome::Delete()
{
        delete window;
}

PacketListWindow::PacketListWindow()
{
  Init();
}

PacketListWindow::~PacketListWindow()
{
        delete packetList;
}

void PacketListWindow::Init()
{
        char tmp[512];

        packetList = new file_list(bm.resourceObject->get(PacketDir));

        if (!NumOfItems()) {
                sprintf(tmp, "No packets found.\n\n"
                        "Please place any packets to be read in:\n%s",
                        bm.resourceObject->get(PacketDir));
                fatalError(tmp);
        }
}

void PacketListWindow::MakeActive()
{
        welcome.MakeActive();
        oldPos = oldActive = -100;
        const int stline = 9;
        list_max_y = LINES - (stline + 9);
        list = new InfoWin(list_max_y + 3, 60, stline, COLS / 2 - 30, C_PLBORDER);
        list->attrib(C_PLHEADER);
        list->putstring(1, 3, "Packet                Size    Date");
        list->delay_update();
        Draw();
}

int PacketListWindow::NumOfItems()
{
        return packetList->getNoOfFiles();
}

void PacketListWindow::Delete()
{
        delete list;
        welcome.Delete();
}

void PacketListWindow::oneLine(int i)
{
        char *tmp = list->lineBuf;

        packetList->gotoFile(position + i);

        const char *tmp2 = packetList->getName();
        time_t tmpt = packetList->getDate();

        strcpy(tmp, "          ");

        for (int j = 2; (*tmp2 != '.') && (j < 10); j++)
                tmp[j] = *tmp2++;

        sprintf(&tmp[10], "%-4.4s%16u  %s", tmp2,
                (unsigned) packetList->getSize(), ctime(&tmpt));

        strcpy(&tmp[56], "  ");

        DrawOne(i, C_PLIST);
}

void PacketListWindow::Select()
{
        packetList->gotoFile(active);
}

void PacketListWindow::ReList()
{
        if (position > 1) {
                if (active > 1)
                        active--;
                position--;
        }
        delete packetList;
        Init();
}

void PacketListWindow::extrakeys(int key)
{
        switch (key) {
        case 'A':
                interface->newstate(address);
                break;
        case 'K':
                char tmp[128];

                Select();
                sprintf(tmp, "Do you really want to delete %s?",
                        packetList->getName());
                if (interface->WarningWindow(tmp) > 0) {
                        mychdir(bm.resourceObject->get(PacketDir));
                        remove(packetList->getName());
                        ReList();
                }
                interface->changestate(packetlist);
                break;
        case KEY_LEFT:
        //case KEY2_LEFT:
                Move(UP);
                Draw();
                break;
        case KEY_RIGHT:
        //case KEY2_RIGHT:
                Move(DOWN);
                Draw();
        }
}

int PacketListWindow::OpenPacket()
{
        Select();
        return bm.selectPacket(packetList->getName());
}
