/*
 * blueMail offline mail reader
 * tagline selection, editing

 Copyright (c) 1996 Kolossvary Tamas <thomas@vma.bme.hu>
 Copyright (c) 1998 William McBrine <wmcbrine@clark.net>
 Copyright (c) 1998 Ingo Brueckl <ib@wupperonline.de>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

#include "interfac.h"

tagline::tagline ()
{
  next = NULL;
}

TaglineWindow::TaglineWindow()
{
        nodraw = 1;
        ReadFile();
}

TaglineWindow::~TaglineWindow()
{
        DestroyChain();
}

void TaglineWindow::MakeActive ()
{
  const int m = interface->midpos(4, 23, 24, 22) + 1;
  const int e = interface->endpos(4, 22) + 1;

  nodraw = 0;
  oldActive = oldPos = -100;

  srand((unsigned) time(NULL));

  list_max_y = LINES - 15;
  list = new InfoWin(LINES - 10, COLS - 2, 5, 1, C_TAGBOXBORDER,
                     "Select tagline", C_TAGBOXTTEXT, 5, 1);

  list->attrib(C_TAGBOXBORDER);
  list->horizline(LINES - 14, COLS - 4);
  list->attrib(C_TAGBOXKEYS);
  list->putstring(LINES - 13, 2, "E");
  list->putstring(LINES - 13, m, "R");
  list->putstring(LINES - 12, 2, "L");
  list->putstring(LINES - 12, e, "Q");
  list->attrib(C_TAGBOXDESCR);
  list->putstring(LINES - 13, 3, ": Manual enter tagline");
  list->putstring(LINES - 13, m + 1, ": Random select tagline");
  list->putstring(LINES - 12, 3, ": Reload tagline file");
  list->putstring(LINES - 12, e + 1, ": Don't apply tagline");

  list->delay_update();
  Draw();
}

void TaglineWindow::Delete()
{
        delete list;
        nodraw = 1;
}

void TaglineWindow::extrakeys(int key)
{
        switch (key) {
        case '\r':
        case '\n':
                if (NumOfTaglines > 0)
                        interface->letterwindow.set_Tagline(
                                getTextOf(active));
                break;
        case 'E':
                EnterTagline();
                break;
        case 'L':
                ReadFile();
                break;
        case 'R':
                RandomTagline();
                break;
        case KEY_RIGHT:
        //case KEY2_RIGHT:
                Move(DOWN);
                Draw();
                break;
        case KEY_LEFT:
        //case KEY2_LEFT:
                Move(UP);
                Draw();
        }
}

void TaglineWindow::RandomTagline()
{
        int i = rand() / (RAND_MAX / NumOfTaglines);

        Move(HOME);
        for (int j = 1; j <= i; j++)
                Move(DOWN);
        Draw();
}

void TaglineWindow::EnterTagline(const char *tag)
{
        FILE *fd;
        char newtagline[TAGLINE_LENGTH + 1];
        int y;

        Move(END);
        if (NumOfTaglines >= list_max_y) {
                y = list_max_y;
                position++;
        } else
                y = NumOfTaglines + 1;
        active++;

        if (!nodraw)
                Draw();
        else {
                list = new InfoWin(5, COLS - 2, 10, 1, C_TAGBOXBORDER);
                list->attrib(C_TAGBOXTTEXT);
                list->putstring(1, 1, "Enter new tagline:");
                list->delay_update();
        }

        strcpy(newtagline, tag ? tag : "");

        if (list->getstring((nodraw) ? 2 : y, 1, newtagline,
                TAGLINE_LENGTH, C_TAGBOXINP, C_TAGBOXLIST)) {

                char *p;
                for (p = &newtagline[strlen(newtagline) - 1];
                     (p > newtagline) && (*p == ' '); p--);     //crop spaces

                *++p = '\0';

                if (newtagline[0]) {
                        tagline *curr = &head;
                        //check dupes
                        while (curr && strcmp(newtagline, curr->text))
                                curr = curr->next;
                        if (!curr)
                                if ((fd = fopen(bm.resourceObject->
                                get(TaglineFile), "at"))) {
                                        fputs(newtagline, fd);
                                        fputc('\n', fd);
                                        fclose(fd);
                                }
                        ReadFile();
                }
        }
        Move(END);

        if (!nodraw)
                Draw();
         else
                list->delay_update();
}

void TaglineWindow::ReadFile()
{
        FILE *fd;

        if ((fd = fopen(bm.resourceObject->get(TaglineFile), "rt"))) {
                DestroyChain();
                NumOfTaglines = position = active = 0;

                tagline *curr = &head;
                while (fgets(curr->text, TAGLINE_LENGTH, fd)) {
                        char *p = curr->text + strlen(curr->text) - 1;
                        if (*p == '\n')
                                *p = '\0';
                        curr->next = new tagline;
                        curr = curr->next;
                        NumOfTaglines++;
                }
                fclose(fd);
        }
}

void TaglineWindow::DestroyChain()
{
        tagline *tmp, *curr = head.next;

        while (curr) {
                tmp = curr;
                curr = curr->next;
                delete tmp;
        }
        head.next = NULL;
}

void TaglineWindow::oneLine(int i)
{
        sprintf(list->lineBuf, "%-76.76s", (position + i > NumOfTaglines) ?
                " " : getTextOf(position + i));

        DrawOne(i, C_TAGBOXLIST);
}

int TaglineWindow::NumOfItems()
{
        return NumOfTaglines;
}

const char *TaglineWindow::getTextOf(int num)
{
        tagline *curr = &head;

        for (int i = 1; (i <= num) && (i < NumOfTaglines); i++)
                curr = curr->next;
        return curr->text;
}
